#!/bin/bash
# vim:sw=4
#-------------------------------------------------------------------------------
# Configuration script for Helix GitConnector.
# Copyright 2016, Perforce Software Inc. All rights reserved.
#

#-------------------------------------------------------------------------------
# Global variables
#-------------------------------------------------------------------------------
MONOCHROME=false
DEBUG=false
PLATFORM=unknown
SSHD_AUTHORIZEDKEYSCOMMAND=false

#-------------------------------------------------------------------------------
# Dependency versions
#-------------------------------------------------------------------------------

# Helix server versions
P4D_MIN_VERSION_CHANGE="1515697"
P4D_MIN_VERSION="2017.2"
P4D_AUTHORIZEDKEYSCOMMAND_MIN_VERSION="2018.1"
P4D_AUTHORIZEDKEYSCOMMAND_MIN_CHANGE="1654951"

# Git >= 1.8.3
GIT_MIN_ACCEPTABLE_VERSION="1.8.3"
GIT_MIN_VERSION="1.8.5"

# OpenSSH >= 6.9
OPENSSH_MIN_VERSION="6.9"

#-------------------------------------------------------------------------------
# Functions
# Note: functions are sorted alphabetically:
#
# check_perforce_server()
# check_perforce_super_user()
# configure_crontab()
# configure_sshd_config()
# configure_syslog()
# create_gc_ssh_run_user()
# create_gc_user()
# create_graph_depot()
# create_sys_run_group()
# debug()
# detect_auth()
# detect_local_p4d()
# die()
# done_all_msgs()
# ensure_64bit()
# ensure_dependencies()
# ensure_root()
# error()
# errorOn()
# find_p4()
# gather_existing_p4d_configuration()
# get_distro()
# get_list_of_existing_graph_depots()
# highlight()
# highlightOff()
# highlightOn()
# prompt_for_graph_depot()
# prompt_for_mindiskspace()
# prompt_for_HTTPS_config()
# prompt_for_SSH_config()
# promptfor()
# read_config()
# register_service()
# report_pubkeyauthentication()
# strip_protocol_from_p4port()
# strong_password()
# update_gc_ssh_run_user()
# upgrade()
# usage()
# validate_depotname()
# validate_mindiskspace()
# validate_nonempty()
# validate_p4port()
# validate_p4user()
# validate_system_user()
# validate_yesno()
# write_config()
#
#-------------------------------------------------------------------------------

# Checks that Helix server is running.
# If SSL, runs p4 trust
# If unicode, sets P4CHARSET to UTF8
# If case sensitive, sets P4CASEHANDLING to sensitive
check_perforce_server()
{
    # Exit fast if we don't have the details we need
    if [ -z "$P4PORT" ]; then
        $hideErrors || error "No P4PORT specified"
        return 1
    fi

    local SSLRE="^ssl"
    local BITS=(${P4PORT//:/ })
    if [[ ${BITS[0]} =~ $SSLRE ]]; then
        debug "Trusting Helix server [$P4PORT]"
        P4TRUST="$P4TRUSTPATH" p4 -c "$P4CLIENT"  -p "$P4PORT" trust -f -y >/dev/null 2>&1
        if [ $? -ne 0 ] || [ ! -f "$P4TRUSTPATH" ]; then
            error "Unable to trust the server [$P4PORT]"
            return 1
        fi
    fi

    local P4INFO=""
    if ! P4INFO=$(p4 -p "$P4PORT" -ztag info 2>/dev/null); then
        $hideErrors || error "Unable to connect to Helix server [$P4PORT]"
        return 1
    fi

    local P4D_VERSION="$(echo "$P4INFO" | grep -F "... serverVersion" | \
        sed -e "s;^.*P4D/[^/]*/;;" -e "s;[^0-9.]*/;/;" -e "s;[.]/;/;" -e "s; .*$;;" -e "s;/; ;" )"
    local P4D_REL="$(echo "$P4D_VERSION" | awk '{ print $1; }')"
    local P4D_CHANGE="$(echo "$P4D_VERSION" | awk '{ print $2; }')"
    local MIN_CHANGE=$P4D_MIN_VERSION_CHANGE
    debug "Helix server version: [$P4D_REL]/[$P4D_CHANGE]"
    debug "For Helix server version '$P4D_REL' the required minimum patch='$MIN_CHANGE'"
    if [ "$(awk 'BEGIN{ if ("'$P4D_REL'" < "'$P4D_MIN_VERSION'") print(1); else print(0) }')" -eq 1 ] || \
       [ -n "$MIN_CHANGE" -a "$P4D_CHANGE" -lt "${MIN_CHANGE:-0}" ]; then
        $hideErrors || error "This Helix server $P4D_REL/$P4D_CHANGE is not supported by GitConnector."
        if [ -n "$MIN_CHANGE" -a "$P4D_CHANGE" -lt "${MIN_CHANGE:-0}" ]; then
            $hideErrors || error "GitConnector supports Helix server version '$P4D_REL' starting with patch='$MIN_CHANGE'"
        else
            $hideErrors || error "GitConnector supports Helix servers starting with [$P4D_MIN_VERSION]/[${P4D_MIN_VERSION_CHANGE}]"
        fi
        return 1
    fi
    if [ "$(awk 'BEGIN{ if ("'$P4D_REL'" >= "'$P4D_AUTHORIZEDKEYSCOMMAND_MIN_VERSION'") print(1); else print(0) }')" -eq 1 ] && \
       [ "$P4D_CHANGE" -ge "$P4D_AUTHORIZEDKEYSCOMMAND_MIN_CHANGE" ]; then
        check_sshd_support
    else
        highlight " AuthorizedKeysCommand not supported for change $P4D_REL / $P4D_CHANGE : must be greater than  $P4D_AUTHORIZEDKEYSCOMMAND_MIN_VERSION / $P4D_AUTHORIZEDKEYSCOMMAND_MIN_CHANGE"
    fi

    highlight "check_perforce_server: SSHD_AUTHORIZEDKEYSCOMMAND = $SSHD_AUTHORIZEDKEYSCOMMAND"

    if echo "$P4INFO" | grep -q -F "unicode enabled"; then
        P4CHARSET="utf8"
        debug "[$P4PORT] server is unicode enabled, setting, P4CHARSET=$P4CHARSET"
    fi

    if echo "$P4INFO" | grep -q -F "caseHandling sensitive"; then
        P4CASEHANDLING="sensitive"
        debug "[$P4PORT] is case-sensitive"
    fi
    P4_SERVICETYPE="$(echo "$P4INFO" | grep serverServices | awk '{print $3}' | xargs)"
    # check if service tpe is a non-forwarding replica
    for st in replica build-server standby; do
        if [[ "$st" == "$P4_SERVICETYPE" ]]; then
            error "The 'serverServices' for the Helix server on P4PORT=[$P4PORT] is '$P4_SERVICETYPE'."
            error "GitConnector may not be configured on a Helix server with services of this type."
            error "Stopping."
            return 1
        fi
    done
}

# Check for sshd support.
check_sshd_support()
{
    local sshd_config=/etc/ssh/sshd_config
    SSHD_AUTHORIZEDKEYSCOMMAND=false
    highlight "check_ssd_support"
    if [ -e $sshd_config ]; then
	highlight "$sshd_config exists. Can configure AuthorizedKeys for GCONN."
	# Example: extracting 6.6 from "OpenSSH_6.6.1p1 Ubuntu-2ubuntu2.8, OpenSSL 1.0.1f 6 Jan 2014"
	SSHD_VERSION="$(/usr/bin/ssh -V 2>&1| cut -d ' ' -f 1| grep -oE '[0-9]+\.[0-9]+')"
	if [[ -n "$SSHD_VERSION" ]] ; then
	    highlight "Detected OpenSSH sshd version = $SSHD_VERSION"
	    versions=${SSHD_VERSION}$'\n'${OPENSSH_MIN_VERSION}
	    # numeric sort on each '.' delimited field - do not expect more than 4 fields
	    sortedV=$( echo "$versions" | sort -t '.' -k 1,1 -k 2,2 -k 3,3 -k 4 -g)
	    readarray -t sortedVA <<<"$sortedV"
	    # If first sorted version is the minimum, good to go.
	    if [[ ${sortedVA[0]} == $OPENSSH_MIN_VERSION ]]; then 
		SSHD_AUTHORIZEDKEYSCOMMAND=true
	    else
		highlight "sshd: unable to configure AuthorizedKeysCommand: version $SSHD_VERSION is < mimumum version $OPENSSH_MIN_VERSION."
	    fi
	fi
    else
	highlight "$ssh_config does not exist. Cannot configure AuthorizedKeys for GCONN. Using cron."
    fi
}

# Checks that the user credentials provided are valid and that the user is
# as super-user.
check_perforce_super_user()
{
    # Exit fast if we don't have the details we need
    if [ -z "$P4PORT" -o -z "$P4USER" ]; then
        $hideErrors || error "No P4PORT or P4USER specified"
        return 1
    fi

    debug "Check [$P4USER]/[$P4PASSWD] against [$P4PORT]"

    if [[ -z "$P4PASSWD" ||  "$P4PASSWD" =~ ^[[:blank:]]+$ ]]; then
        echo "P4PASSWD='$P4PASSWD' is empty or is whitespace. Skipping Helix server login."
    else
        if ! echo "$P4PASSWD" | p4 -c "$P4CLIENT" -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" login >/dev/null 2>&1; then
            $hideErrors || error "Unable to login to the Helix server '$P4PORT' as '$P4USER' with supplied password"
            return 1
        fi
    fi

    # Check the output of p4 protects -m
    local protects_out=$(mktemp)
    if ! p4 -c "$P4CLIENT" -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" protects -m 2> "$protects_out" | grep -q "super"; then
        $hideErrors || error "User '$P4USER' must have super privileges"
        $hideErrors || cat "$protects_out"
        return 1
    fi
    rm "$protects_out"

    return 0
}

# Configure CRONTAB for GitConnector
configure_crontab()
{
    highlight "Configuring crontab"

    if [ ! -d /etc/cron.d/ ]; then
        mkdir /etc/cron.d
    fi

    if [ -n "$CRON_INTERVAL" ]; then
        # 0 means crontab will not be created or will be disabled if it exists
        if [[ "$CRON_INTERVAL" =~ ^[0-9]+$ ]] && [ "$CRON_INTERVAL" -eq 0 ]; then
            highlight "Specified cron interval is $CRON_INTERVAL."
            highlight "Crontab will not be created or removed if it already exists."
            if [ -e /etc/cron.d/gconn ]; then
                rm -rf /etc/cron.d/gconn
                highlight "Existing crontab /etc/cron.d/gconn removed."
            fi
            return
        # Check that CRON_INTERVAL is a valid number between 1 and 60
        elif ! [[ "$CRON_INTERVAL" =~ ^[0-9]+$ ]] ||\
            [ "$CRON_INTERVAL" -lt 1 ] || [ "$CRON_INTERVAL" -gt 60  ]; then
            error "$CRON_INTERVAL is not a number between 1 and 60. "
            error "Crontab interval will be set to 10 minutes."
            unset CRON_INTERVAL
        fi
    fi

    # If cron interval is not set, default to 10 minutes
    UPDATE_INTERVAL=${CRON_INTERVAL:-10}

    if [ -e /etc/cron.d/gconn ]; then
        sed -i -e "s;^GCONN_CONFIG.*$;GCONN_CONFIG = $GCONNCONFIGPATH;g" /etc/cron.d/gconn
        sed -i -e "s;^# update auth.*$;# update auth keys every $UPDATE_INTERVAL minute(s);g" /etc/cron.d/gconn
	# Delete all (only one) cron entries for sync-ssh-keys. YAGNI -  one gconn instance per host.
        sed -i -e "/^$/ d" -e "/.* .* .* .* .*sync-ssh-keys/ d" /etc/cron.d/gconn
        {
            echo "*/$UPDATE_INTERVAL * * * * $SSH_USER /usr/bin/gconn sync-ssh-keys"
            echo ""
        } >> /etc/cron.d/gconn
    else
        {
            echo "GCONN_CONFIG = $GCONNCONFIGPATH"
            echo "# CAUTION: This file is updated by $SCRIPT"
            echo "# update auth keys every $UPDATE_INTERVAL minute(s)"
            echo "*/$UPDATE_INTERVAL * * * * $SSH_USER /usr/bin/gconn sync-ssh-keys"
            echo ""
        } > /etc/cron.d/gconn
    fi
}

# Configure /etc/ssh/sshd_config for
configure_sshd_config()
{
    if [ "$SSH" == 'true' ] && [ -n "$SSH_USER" ]; then
        sshd_config=/etc/ssh/sshd_config
        if [ -e $sshd_config ]; then
            if ! grep "^Match User $SSH_USER" $sshd_config > /dev/null; then   
                highlight "Adding 'Match User $SSH_USER' / AuthorizedKeysCommand to $sshd_config"
                {
                    echo "Match User $SSH_USER"
                    echo "AuthorizedKeysCommand $GCONNSSHAUTHSCRIPT %t %k"
                    echo "AuthorizedKeysCommandUser $SSH_USER"
                } >> $sshd_config
                write_ssh_auth_script
                service ssh restart
                # remove cron for: gconn sync_ssh_keys
                if [ -e /etc/cron.d/gconn ]; then
                    rm -rf /etc/cron.d/gconn
                    highlight "Existing crontab /etc/cron.d/gconn removed."
                fi
            else
                highlight "$sshd_config: 'Match User $SSH_USER' already exists. Nothing to do."
            fi
        fi
    fi

}

# Configure SYSLOG for GitConnector
configure_syslog()
{
    highlight "Configuring logging"
    if ! $RUN_AS_SUDO ; then
        local conf=/etc/rsyslog.d/30-gconn.conf   
        if [ ! -f $conf ] ; then
            highlight "Warning: $conf does not exist. Rerun as sudo in order to create."
        fi
        highlight "Warning: Not configuring logging. Rerun as sudo to do so."
        return
    fi

    # Detect where the syslog logrotate configuration is
    local LOGROTATE_CFG=""
    if [ -e "/etc/logrotate.d/rsyslog" ]; then
        LOGROTATE_CFG="/etc/logrotate.d/rsyslog"
    elif [ -e "/etc/logrotate.d/syslog" ]; then
        LOGROTATE_CFG="/etc/logrotate.d/syslog"
    else
        error "Failed to find logrotate configuration! Skipping syslog configuration!"
        return
    fi

    local LOGPATH="/var/log"
    # Add the extra syslog filter rules for GitConnector
    mkdir -p /etc/rsyslog.d
    {
        echo ":syslogtag,contains,\"Perforce-gconn\" -$LOGPATH/gconn.log"
        echo ":syslogtag,contains,\"Perforce-p4gc\" -$LOGPATH/p4gc.log"
	echo "\$EscapeControlCharactersOnReceive off"
    } > /etc/rsyslog.d/30-gconn.conf

    # Replace previous Git-connector log rotation configuration
    sed "\|$LOGPATH/gconn|d" -i "$LOGROTATE_CFG"
    sed "\|$LOGPATH/p4gc|d" -i "$LOGROTATE_CFG"
    {
        echo "$LOGPATH/gconn.log"
        echo "$LOGPATH/p4gc.log"
        cat "$LOGROTATE_CFG"
    } > /tmp/syslog.tmp
    cat /tmp/syslog.tmp > "$LOGROTATE_CFG"
    rm -f /tmp/syslog.tmp

    # We'll try the standard way to restart the daemon first
    if ! /etc/init.d/rsyslog restart >/dev/null 2>&1; then
        # If that failed (or at least said it failed) try the service command
        service rsyslog restart >/dev/null 2>&1
    fi
}

# Creates a new GConn SSH RUN USER
# Usage: create_gc_ssh_run_user <username>
create_gc_ssh_run_user()
{
    local USER="$1"

    highlight "Creating user [$USER]"
    # Create the new user (with an empty home directory)
    mkdir -p "$SSH_USER_HOME"
    useradd --system --home "$SSH_USER_HOME"\
            --shell /bin/bash -c "Helix GitConnector" \
            -g "$RUN_GROUP" "$USER"
    chown -R "$USER":"$RUN_GROUP" "$SSH_USER_HOME"
    highlight "User [$USER] created!"
}

create_gc_user()
{
    highlight "Creating/updating GitConnector user [$GCUSER] account"
    if ! p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" users -a "$GCUSER"  2>&1 | grep 'no such user' > /dev/null; then
        debug "[$GCUSER] already exists"
        if [ -n "$GCUSER_PASSWD" ] && [ -z "$AUTH_LDAP" ]; then
            debug "Changing password for [$GCUSER]"
            p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" passwd "$GCUSER" > /dev/null <<!
$GCUSER_PASSWD
$GCUSER_PASSWD
!
        fi
    else
        debug "$GCUSER does not exist, creating"
        debug "Running p4 user for [$GCUSER]"
        p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" user -o -f "$GCUSER" |\
            p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" user -i -f
        # Do not set password if auth method is set to LDAP
        if [ -z "$AUTH_LDAP" ]; then
            debug "Running p4 passwd for [$GCUSER]"
            p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" passwd "$GCUSER" > /dev/null <<!
$GCUSER_PASSWD
$GCUSER_PASSWD
!
        fi
    fi

    highlight "Creating/updating group [$GCUSER_GROUP] for [$GCUSER]"
    p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" group -o "$GCUSER_GROUP" | \
          sed 's/^Timeout:.*$/Timeout: unlimited/' | \
          awk "/^Users:/{print;print \"\t$GCUSER\";next}1" | \
          p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" group -i

    debug "Logging in [$GCUSER]"
    p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" login "$GCUSER" > /dev/null

    if [ ! -f "$P4TICKETSPATH" ]; then
       echo "Unable to create $P4TICKETSPATH for $GCUSER"
       exit 1
    fi

    highlight "Setting required permissions for [$GCUSER] to access Graph depots"
    p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" grant-permission -d \* -u "$GCUSER" -p admin > /dev/null
    debug "[$GCUSER] updated"
}

create_graph_depot()
{
    if [ -z "$GRAPHDEPOT" ]; then
        debug "No new Git depot specified to be created"
        return
    fi
    # validate_depotname checks if $GRAPHDEPOT already exists, and is a valid depot name
    if validate_depotname $GRAPHDEPOT; then
        highlight "Creating Git depot [$GRAPHDEPOT]"
        if  ! p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" depot -o -t graph "$GRAPHDEPOT" |\
                 p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" depot -i; then
            error "Failed to create Git depot [$GRAPHDEPOT]!"
            return 1
        fi
        GRAPH_DEPOTS=("${GRAPH_DEPOTS[@]}" "$GRAPHDEPOT")
    else
        highlight "Git depot [$GRAPHDEPOT] will not be created."
    fi
}

# Create group $RUN_GROUP to be shared between Apache and SSH_USER ('git')
create_sys_run_group()
{
    # Check if group already exists
    if ! getent group "$RUN_GROUP" >/dev/null 2>&1 ; then
        debug "Creating system group [$RUN_GROUP]"
        if ! groupadd "$RUN_GROUP"; then
            die "Failed to create system group [$RUN_GROUP]"
        fi
    fi
    debug "Changing file permissions for system group [$RUN_GROUP]"
    chmod 440 "$P4TICKETSPATH"
    chgrp "$RUN_GROUP" "$P4TICKETSPATH"
    if [ -e "$P4TRUSTPATH" ]; then
        chmod 440 "$P4TRUSTPATH"
        chgrp "$RUN_GROUP" "$P4TRUSTPATH"
    fi
    if [ ! -d "$REPOSDIR" ]; then
        mkdir "$REPOSDIR"
    fi
    chmod 770 "$REPOSDIR"
    chgrp "$RUN_GROUP" "$REPOSDIR"
    chmod g+s "$REPOSDIR"
    debug "System group [$RUN_GROUP] is created"

}

debug()
{
    $DEBUG || return 0
    $MONOCHROME || echo -e "\033[33m"$1"\033[0m" >&2
    $MONOCHROME && echo -e "$1" >&2 || true
}

detect_auth()
{
    debug "Checking if authentication method is LDAP"
    local AUTH_METHOD="$(p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" -ztag configure show auth.default.method | grep Value |  awk '{print $3}')"
    local P4INFO_LDAP="$(p4 -p "$P4PORT" -ztag info 2>/dev/null | grep ldap )"
    local GCUSER_AUTH=""
    if ! p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" users -a "$GCUSER"  2>&1 | grep 'no such user' > /dev/null; then
        debug "[$GCUSER] already exists, checking if its' auth method is LDAP"
        GCUSER_AUTH="$(p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" user -o $GCUSER | grep AuthMethod |  awk '{print $3}')"
    fi
    if [ "$AUTH_METHOD" = "ldap" ] || [ -n "$P4INFO_LDAP" ] || [ "$GCUSER_AUTH" = "ldap" ]; then
        highlight "$SCRIPT detected LDAP authentication."
        highlight "Please set or change password for [$GCUSER] (if needed) manually."
        AUTH_LDAP="true"
    fi
}

# Attempt to determine if there is a local Helix server using p4dctl
detect_local_p4d()
{
    # Don't attempt to use p4dctl if it is not available
    if $INTERACTIVE && which p4dctl >/dev/null 2>&1; then
        hideErrors=false
        saved_port=$P4PORT
        saved_user=$P4USER
        saved_passwd=$P4PASSWD
        debug "Using p4dctl to detect if there is a local Helix server."
        local DEFAULT_SERVICE="$(p4dctl list -t p4d 2>&1 | sed -n '2p' | awk '{ print $3; }')"
        if [ -n "$DEFAULT_SERVICE" ]; then
            eval "$(p4dctl env $DEFAULT_SERVICE -t p4d P4PORT)"
            eval "$(p4dctl env $DEFAULT_SERVICE -t p4d P4USER)"
            if validate_p4port "$P4PORT"; then
                highlight "There appears to be a local Helix server [$DEFAULT_SERVICE] on P4PORT [$P4PORT]."
                promptfor USE_DETECTED_HELIX "Would you like to use this configuration for GitConnector? [yes/no]" "yes" false validate_yesno
                if [ "$USE_DETECTED_HELIX" == 'yes' ]; then
                    if [ -z "$P4USER" ]; then
                        promptfor P4USER "Enter the Helix super-user's username" "$P4USER" false validate_p4user
                        promptfor P4PASSWD "Enter the Helix super-user's password" "$P4PASSWD" true
                    elif validate_p4user "$P4USER"; then
                        highlight "The super-user for the Helix server appears to be [$P4USER]."
                        promptfor USE_DETECTED_P4USER "Would you like to use this P4USER as super-user for GitConnector? [yes/no]" "yes" false validate_yesno
                        if [ "$USE_DETECTED_P4USER" == 'yes' ]; then
                            promptfor P4PASSWD "Enter the Helix super-user's password" "$P4PASSWD" true
                        fi
                    fi
                else
                    highlight "Detected Helix server will not be used. You will be prompted for P4PORT and P4USER later."
                    P4PORT=$saved_port
                    P4USER=$saved_user
                    return
                fi
            else
                highligh "Detected Helix server will not be used. You will be prompted for P4PORT and P4USER later."
                P4PORT=$saved_port
                P4USER=$saved_user
            fi
        else
            highlight "No local Helix server detected, continuing with configuration."
            return
        fi
        # Just one chance to pass right values since users will be prompted for them later in case of a mistake
        if ! check_perforce_server  || ! check_perforce_super_user ; then
            highlight "Detected Helix server will not be used. You will be prompted for P4PORT and P4USER later."
            P4PORT=$saved_port
            P4USER=$saved_user
            P4PASSWD=$saved_passwd
        else
            # Success! Running Helix service is detected, use supplied P4PORT, P4USER and P4PASSWD
            DETECTED_HELIX='true'
        fi
    fi
}

die()
{
    error "FATAL: $@" >&2
    exit 1
}

done_all_msgs()
{
    highlightOn
    cat <<EOT

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
::  GitConnector configuration has completed successfully.
::
::  Here is what had been done so far:
::    - GitConnector's configuration file is in [$GCONNCONFIGPATH]
::    - Helix user for GitConnector is [$GCUSER]
::    - [$GCUSER] belongs to Helix group [$GCUSER_GROUP] which has
::      an unlimited ticket timeout
EOT
    if $REGISTERED_SERVICE; then
        echo "::    - This GitConnector was registered with Helix server as [$SERVER_ID]"
    fi
    if [ -n "$GRAPH_DEPOTS" ]; then
        echo "::    - GitConnector configured with Git depot(s):"
    for depot in "${GRAPH_DEPOTS[@]}"
    do
        echo "::      [$depot]"
    done
    fi
    if $HTTPS && ! $HTTPS_FAIL; then
    cat <<EOT
::    - GitConnector is configured for HTTPS authentication
::    - The hostname of this GitConnector is [$GCONN_HOSTNAME]
EOT
    fi
    if $SSH && ! $SSH_FAIL; then
    cat <<EOT
::    - GitConnector is configured for SSH authentication with system
::      user [$SSH_USER]
::    - HOME directory of system user [$SSH_USER] is [$EXISTING_SSH_USER_HOME]
EOT
        if [ -n "$CRON_INTERVAL" ]; then
            if [ "$CRON_INTERVAL" -eq 0 ]; then
            cat <<EOT
::    - Crontab to update SSH keys was not installed or is disabled
::
EOT
            fi
        fi
        if [ -n "$UPDATE_INTERVAL" ]; then
        cat <<EOT
::    - SSH keys will be updated every $UPDATE_INTERVAL minute(s)
::
EOT
        fi
    fi
    if $HTTPS_FAIL || $GIT_WARN || $SSH_FAIL || ! $REGISTERED_SERVICE ; then
    cat <<EOT
::
::  Here is what went wrong:
EOT
        if $HTTPS_FAIL; then
        cat <<EOT
::    - GitConnector could not configure HTTPS authentication
::      HTTPS authentication is supported only on Ubuntu 14.04 and later
::      and CentOS/RedHat 6 and later
EOT
            if ! $DEBUG; then
            cat <<EOT
::      Please re-run this script with --debug flag to get more detailed output
EOT
            fi
        fi
        if $GIT_WARN; then
        cat <<EOT
::    - GitConnector detected that your version of Git [$GIT_VER] is less than
::      recommended minimum Git version [$GIT_MIN_VERSION]. If possible, please
::      install a newer version of Git
EOT
        fi
        if $SSH_FAIL; then
        cat <<EOT
::    - GitConnector could not configure SSH authentication
EOT
        fi
        if ! $REGISTERED_SERVICE; then
          cat <<EOT
::    - This GitConnector service was not registered with Helix server. Service
::      registration is required for correct operation of some features, such as
::      mirroring and repo polling, which now might not work as intended.
EOT
        fi
    fi
    cat <<EOT
::
::  Here is what still needs to be done:
EOT
    if [ -n "$AUTH_LDAP" ]; then
    cat <<EOT
::   - This script detected LDAP authentication. Please create or set password
::     for [$GCUSER] P4USER manually (if needed) and log in as [$GCUSER] user using
::     the script [$RELOGINSCRIPT]
EOT
    fi
    if [ "$SSH" == 'true' ] && ! $SSH_FAIL; then
    cat <<EOT
::    - Git users need to add SSH public keys using command:
::       p4 -p $P4PORT pubkey -i < /path/to/ssh/key.pub
EOT
    fi
    cat <<EOT
::    - Grant Git users permissions to create repos with command:
EOT
if [ -n "$GRAPH_DEPOTS" ]; then
    for depot in "${GRAPH_DEPOTS[@]}"
    do
        echo "::      p4 -p $P4PORT -u $P4USER grant-permission -p create-repo -d $depot -u <username>"
    done
else
    echo "::      p4 -p $P4PORT -u $P4USER grant-permission -p create-repo -d <depotname> -u <username>"
fi
cat <<EOT
::    - In case you need to manually log in [$GCUSER], please use login
::      script in [$RELOGINSCRIPT]
::
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

EOT
    highlightOff
}

# Architecture check
# GitConnector only runs on 64-bit systems (amd64 or x86_64)
# uname -p is more specific, but Debian returns 'unknown'
ensure_64bit()
{
    if ! uname -a | grep -q -E "x86_64|amd64"; then
        error "GitConnector only supports x86_64 based architectures."
        exit 1
    fi
}

# Dependency check
ensure_dependencies()
{
    debug "Checking dependencies"
    # Git
    if which git >/dev/null 2>&1; then
        GIT_VER="$(git --version | grep -o '[0-9.]*')"
        local GIT_VER_ARR=(${GIT_VER//./ })
        local GIT_MIN_ACCEPT_ARR=(${GIT_MIN_ACCEPTABLE_VERSION//./ })
        local GIT_MIN_ARR=(${GIT_MIN_VERSION//./ })
        for (( i=0 ; i < ${#GIT_MIN_ARR[@]} ; i++ )); do
            local min=${GIT_MIN_ARR[i]}
            local min_acc=${GIT_MIN_ACCEPT_ARR[i]}
            local current=${GIT_VER_ARR[i]}
            if [ -z "$current" ] || [ "$current" -lt "$min_acc" ]; then
                highlightOn
                echo "Current Git version is [$GIT_VER] but GitConnector requires at least version [$GIT_MIN_ACCEPTABLE_VERSION]."
                highlightOff
                die "Please install a newer version of Git. Minimum recommended version of Git is [$GIT_MIN_VERSION]."
            elif [ "$current" -lt "$min" ]; then
                GIT_WARN=true
                highlightOn
                echo "Current Git version is [$GIT_VER], but GitConnector recommends Git version [$GIT_MIN_VERSION] or higher."
                highlightOff
            elif [ "$current" -gt "$min_acc" ]; then
                break
            fi
        done
    else
        die "Can't find git! : $PATH"
    fi

    if [ ! -e "$GCONNINSTALLPATH"/bin/gconn ]; then
        die "Git Connector is not installed with packages. Please install helix-git-connector package."
    fi
}

# Root user check
# If not root, escalate with sudo
ensure_root()
{
    if [ $EUID != 0 ]; then
        error "This script must run with root privileges. Attempting to sudo!"
        sudo -H "$SCRIPTPATH/$SCRIPT" "$@"
        exit $?
    else
        # If we are root from a previous sudo, we need to ensure that the home
        # direectory in the environment is correct for the effective user
        if [ "$HOME" != "$(getent passwd "$EUID" | cut -d ":" -f 6 )" ]; then
            export HOME="$(getent passwd "$EUID" | cut -d ":" -f 6 )"
        fi
    fi
}

error()
{
    $MONOCHROME || echo -e "\033[31m"$1"\033[0m" >&2
    $MONOCHROME && echo -e "$1" >&2 || true
}

errorOn()
{
    $MONOCHROME || echo -e "\033[31m"
    $MONOCHROME && echo "" || true
}

# Got P4?
find_p4()
{
    debug "Check that command-line client is installed"
    if ! which p4 >/dev/null 2>&1; then
        return 1
    fi
    local P4_VER=""
    if P4_VER="$(p4 -V | grep -o 'P4/[^ ]*/[^ ]*/[^ ]*')" >/dev/null 2>&1; then
        P4_VER="$(echo "$P4_VER" | sed -e "s;^.*P4/[^/]*/;;" -e "s;[^0-9.]*/;/;" -e "s;[.]/;/;" -e "s; .*$;;" -e "s;/; ;")"
        local P4_REL="$(echo "$P4_VER" | awk '{ print $1; }')"
    fi
    return $?
}

# Gathers Helix related variables for configuring GitConnector against an
# existing Helix server
gather_existing_p4d_configuration()
{
    debug "Gather existing Helix server configuration"
    # Get and validate the admin credentials
    if $INTERACTIVE && [ -z "$P4PORT" -o -z "$P4USER" ]; then
        hideErrors=true
    fi
    while ! check_perforce_server  || ! check_perforce_super_user; do
        if $INTERACTIVE; then
            hideErrors=false
            promptfor P4PORT "Enter the Helix server's P4PORT" "$P4PORT" false validate_p4port
            promptfor P4USER "Enter the Helix super-user's username" "$P4USER" false validate_p4user
            promptfor P4PASSWD "Enter the Helix super-user's password" "$P4PASSWD" true
        else
            # Detailed errors reported by check functions
            exit 1
        fi
    done
}

# Work out which flavour *nix we're on
get_distro()
{
    debug "Determining platform"
    if [ -e '/etc/redhat-release' ]; then
        PLATFORM="$(</etc/redhat-release)"
    elif [ -e '/etc/SuSE-release' ]; then
        PLATFORM="$(sed -n 's/SUSE = //p' /etc/SuSE-release)"
    elif [ -e '/etc/lsb-release' ]; then
        PLATFORM="$(sed -n 's/DISTRIB_DESCRIPTION=//p' /etc/lsb-release | tr -d '"')"
    else
        if [ "$(uname -o)" != "GNU/Linux" ]; then
            PLATFORM=notlinux
        fi
        # Exit now if this is not a supported Linux distribution
        die "Could not determine OS distribution"
    fi
    debug "Platform is [$PLATFORM]"
}

# Get list of existing Git depots and store them in an array
get_list_of_existing_graph_depots()
{
    GRAPH_DEPOTS=($(p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" depots --depot-type=graph 2>&1 |  cut -d' ' -f2 2>&1))
}

highlight()
{
    $MONOCHROME || echo -e "\033[32m"$1"\033[0m"
    $MONOCHROME && echo -e "$1" || true
}

highlightOff()
{
    $MONOCHROME || echo -e "\033[0m"
    $MONOCHROME && echo "" || true
}

highlightOn()
{
    $MONOCHROME || echo -e "\033[32m"
    $MONOCHROME && echo "" || true
}

prompt_for_graph_depot()
{
    # Do not prompt in non-interactive mode
    if ! $INTERACTIVE; then
        return
    fi

    cat <<EOT
The Helix GitConnector uses one or more depots of type 'graph', aka Git depot.
A Git depot is a container for one or more Git repos, each of which can
contain standard Git files.
A typical Git depot name is 'repo' and a typical repo name is //repo/projectA.
EOT
    if [ -n "$GRAPH_DEPOTS" ]; then
        echo "GitConnector detected existing Git depot(s):"
        for depot in "${GRAPH_DEPOTS[@]}"
        do
            echo "'$depot'"
        done
        # Do not prompt for Git depot name if users already specified one in command line
        if [ -z "$GRAPHDEPOT" ]; then
            promptfor CREATE_GRAPHDEPOT "Would you like to create an additional Git depot? [yes/no]" "no" false validate_yesno
            if [ "$CREATE_GRAPHDEPOT" == "no" ]; then
                return
            fi
        else
            echo "An additional Git depot '$GRAPHDEPOT' will be created."
        fi
    fi
    if ! validate_depotname "$GRAPHDEPOT"; then
        promptfor GRAPHDEPOT "Please provide the desired name for new Git depot" "$GRAPHDEPOT" false validate_depotname
    fi
}

prompt_for_mindiskspace()
{
    # Do not prompt in non-interactive mode
    if ! $INTERACTIVE; then
        return
    fi

    if [ $MIN_DISK_SPACE -ne $DEFAULT_MIN_DISK_SPACE ]; then
        echo "GitConnector detected existing Min_Diskspace setting is:"
        echo "'$MIN_DISK_SPACE'"
    fi
    promptfor MIN_DISK_SPACE "Minimum Disk Space? [1GB or more]" "$MIN_DISK_SPACE" false validate_mindiskspace
}

# Should we configure HTTPS?
prompt_for_HTTPS_config()
{
    # Do not prompt in non-interactive mode
    if ! $INTERACTIVE; then
        return
    fi

    # If users supply --https flag from the command line, do not prompt again
    if $HTTPS && $GCONN_HOSTNAME; then
        return
    fi

    cat <<EOT

Git and GitConnector support the SSH and HTTPS protocols for accessing remote
Git servers. HTTPS support enables the use of web style remote URLs:

  $ git clone https://www.myco.com/graphdepot/myrepo.git

When GitConnector is configured with HTTPS support, users authenticate using
their Helix user name and password, no pre-shared keys are required. This
script installs Apache2 and related packages.
EOT
    promptfor HTTPS_PROMPT "Do you wish to configure GitConnector with HTTPS support? [yes/no]" "yes" false validate_yesno
    if [ "$HTTPS_PROMPT" == "yes" ]; then
        HTTPS=true
        # Prompt for gconn's hostname only if users want to configure HTTPS
        cat <<EOT

GitConnector needs a distinct hostname that users can enter when issuing Git
commmands when accessing GitConnector. Ideally, this is a fully-qualified domain
name, e.g. 'gitconnector.company.com', but it can be just a hostname,
e.g. 'gitconnector'.

Whatever hostname you provide should be GitConnector-specific.

Note that the hostname you specify typically requires configuration in your
network's DNS service. If you are merely testing GitConnector, you can add a
hostname->IP mapping entry to your computer's hosts configuration.

EOT
    promptfor GCONN_HOSTNAME "Please enter hostname for this GitConnector host" "$DEFAULT_GCONN_HOSTNAME" false validate_nonempty
    fi
}

# Should we configure SSH?
prompt_for_SSH_config()
{
    # Do not prompt in non-interactive mode
    if ! $INTERACTIVE; then
        return
    fi

    # If users supply --ssh flag from the command line, do not prompt again
    if $SSH; then
        return
    fi

    cat <<EOT

Git and GitConnector support the SSH and HTTPS protocols for accessing remote
Git servers. SSH support enables the use of ssh style remote URLs:

  $ git clone git@myco.com:graphdepot/myrepo.git

When GitConnector is configured with SSH support, users authenticate using SSH
keys that they generate on their client using the 'ssh-keygen' command.
To add the SSH public key to the Helix server, use the 'p4 pubkey' command.

This installer creates a new operating system user, named 'git' by default.

To support the authentication of new graph depot users, a crontab job updates
SSH keys from p4d. The default update interval is 10 minutes.
EOT
    promptfor SSH_PROMPT "Do you wish to configure GitConnector with SSH support? [yes/no]" "yes" false validate_yesno
    if [ "$SSH_PROMPT" == "yes" ]; then
        SSH=true
        report_pubkeyauthentication
    fi
}

# Prompt the user for information by showing a prompt string, and if the
# prompt is for a password also disabling echo on the terminal. Optionally
# calls a validation function to check if the response is OK.
#
# promptfor <VAR> <prompt> <default> [<ispassword>] [<validationfunc>]
promptfor()
{
    local var="$1"
    local prompt="$2"
    local default="$3"
    local secure=false
    local check_func=true
    local non_default_check_func=false

    [ -n "$4" ] && secure=$4
    [ -n "$5" ] && check_func=$5 && non_default_check_func=true

    if [ "$default" = " " ]; then
        default=""
    fi

    while true; do
        local pw=""
        local pw2=""
        if $secure; then
            stty -echo echonl
        fi

        if [ -n "$default" ]; then
            showDefault=$default
            if [ "$secure" = "true" ]; then
                showDefault=$(echo "$default" | sed 's/./*/g')
            fi
            read -e -p "$prompt [$showDefault]: " pw
            if [ ! -n "$pw" ]; then
                pw=$default
            fi
        else
            read -e -p "$prompt: " pw
        fi
        stty echo -echonl || true
        if $secure; then
            echo ""
        fi
        if $check_func "$pw"; then
            if $secure && $non_default_check_func; then
                stty -echo echonl
                read -e -p "Re-enter new password: " pw2
                stty echo -echonl || true
                echo ""
                if [ "$pw" == "$pw2" ]; then
                    eval "$var=\"$pw\""
                    break;
                else
                    echo "Passwords do not match. Please try again."
                fi
            else
                eval "$var=\"$pw\""
                break;
            fi
        fi
    done
    true
}

read_config()
{
    local CONFIG="$1"
    debug "Reading config [$CONFIG]"

    if [ ! -e "$CONFIG" ]; then
        error "[$CONFIG] does not exist!"
    fi

    # Now attempt to read the config
    # Some of the values could have been passed in the command line,
    # so don't bluntly overwrite those.
    CONF_ITEMS="$( GCONN_CONFIG="$GCONNCONFIGPATH" "$GCONNINSTALLPATH"/bin/gconn show-config )"
    REPOSDIR="$( echo "$CONF_ITEMS" | grep 'reposDir' | cut -d'=' -f 2 )"
    OLD_GCUSER="$( echo "$CONF_ITEMS" | grep 'p4User' | cut -d'=' -f 2 )"
    OLD_P4PORT="$( echo "$CONF_ITEMS" | grep 'p4Port' | cut -d'=' -f 2 )"
    P4TICKETSPATH="$( echo "$CONF_ITEMS" | grep 'p4TicketsFile' | cut -d'=' -f 2 )"
    OLD_MIN_DISK_SPACE="$( echo "$CONF_ITEMS" | grep 'minDiskSpace' | cut -d'=' -f 2 )"
    P4TRUSTPATH="$( echo "$CONF_ITEMS" | grep 'p4TrustFile' | cut -d'=' -f 2 )"
    AUTHKEYSFILEPATH="$( echo "$CONF_ITEMS" | grep 'authKeysFile' | cut -d'=' -f 2 )"
    GIT_PATH="$( echo "$CONF_ITEMS" | grep 'gitExecPath' | cut -d'=' -f 2 )"
    ENVCONFIGPATH="$( echo "$CONF_ITEMS" | grep 'envPath' | cut -d'=' -f 2 )"
    RUN_GROUP="$( echo "$CONF_ITEMS" | grep 'authGroup' | cut -d'=' -f 2 )"
    OLD_SERVER_ID="$( echo "$CONF_ITEMS" | grep 'serverId' | cut -d'=' -f 2 )"
    highlight "Finished reading configuration file. Existing values are:"
    highlight "reposDir is \"$REPOSDIR\""
    highlight "p4User is \"$OLD_GCUSER\""
    highlight "p4Port is \"$OLD_P4PORT\""
    highlight "p4TicketsFile is \"$P4TICKETSPATH\""
    highlight "minDiskSpace is \"$OLD_MIN_DISK_SPACE\""
    highlight "p4TrustFile is \"$P4TRUSTPATH\""
    highlight "authKeysFile is \"$AUTHKEYSFILEPATH\""
    highlight "gitExecPath is \"$GIT_PATH\""
    highlight "envPath is \"$ENVCONFIGPATH\""
    highlight "authGroup is \"$RUN_GROUP\""
    highlight "serverId is \"$OLD_SERVER_ID\""
}

# Register this GConn as a service with p4d
register_service()
{
    debug "Check that we can register this GitConnector as [\"$SERVER_ID\"]"
    if p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" servers | grep $SERVER_ID > /dev/null; then
        highlight "Server with the Server ID [\"$SERVER_ID\"] already exists."
        if $INTERACTIVE; then
            highlight "It is required that each GitConnector is registered with P4D."
            highlight "If you wish to use a different Server ID, please answer [no] at the prompt"
            highlight "and re-run this script with --serverid <serverid> flag."
            promptfor OVERWRITE_SERVER "Do you wish to overwrite existing GitConnector server configuration [yes/no]" "yes" false validate_yesno
            if [ "$OVERWRITE_SERVER" == "no" ]; then
                error "This GitConnector will not be registered with Helix server"
                SERVER_ID="none"
                return
            fi
        elif ! $SERVER_ID_FLAG; then
            error "Please specify desired Server ID with flag --serverid <serverid>"
            error "This GitConnector will not be registered with Helix server"
            SERVER_ID="none"
            return
        fi
    fi
    highlight "Registering this GitConnector [\"$SERVER_ID\"] service with Helix server"
    p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" server -i <<EOT
ServerID: $SERVER_ID
Type: connector
Services: git-connector
User: $GCUSER
Description:
    This GitConnector service was configured on [$(date)] by user [$P4USER].

    Configuration summary:
      Platform: [$PLATFORM]
      HTTPS authentication: [$HTTPS]
      SSH authentication: [$SSH]
      SSH access system user: [$SSH_USER]
EOT
    REGISTERED_SERVICE=true
}

# Report a message if /etc/ssh/ssh_config has disasbled PubkeyAuthentication
report_pubkeyauthentication()
{
    debug "Check that ssh config allows for PubkeyAuthentication"
    ssh_config=/etc/ssh/sshd_config
    kname=PubkeyAuthentication
    if [ -e $ssh_config ] ; then
        kvalue="$(grep $kname $ssh_config | grep -v '#' | awk '{print $2;}' )"
        if [ "$kvalue" == 'no' ]; then
            highlight "Public key authentication is disabled on this system."
            highlight "'$ssh_config' is configured with '$kname $kvalue'."
            highlight "This setting prevents PublicKey authentication for the git ssh protocol."
            highlight "Contact your system administrator in order to enable this feature."
            unset $SSH
        fi
    fi
}

# Use a white list of protocols prefixes to match and strip from a P4PORT
strip_protocol_from_p4port()
{
    p4port="$1"
    for pro in tcp: tcp4: tcp6: tcp46: tcp64: ssl: ssl4: ssl6: ssl46: ssl64:
    do
        if [[ $p4port = $pro* ]]; then
            p4port=${p4port#$pro}
            break
        fi
    done
    echo "$p4port"

}

# Ensure password is Strong by Helix definition
# Usage: strong_password <password>
strong_password()
{
    local pw=$1
    local secure=false

    if [ ${#pw} -ge 8 ]; then
        # Test for two character classes
        if echo "$pw" | egrep '[[:upper:]]' | egrep '[[:lower:]]' >/dev/null; then
            secure=true
        elif echo "$pw" | egrep '[[:upper:]]' | egrep '[^[:alpha:]]' > /dev/null; then
            secure=true
        elif echo "$pw" | egrep '[[:lower:]]' | egrep '[^[:alpha:]]' > /dev/null; then
            secure=true
        else
            error "Password too simple"
        fi
    else
        [ -n "$pw" ] && error "Password too short."
    fi
    $secure
}

# Updates GConn SSH RUN USER
# Usage: update_gc_ssh_run_user <username>
update_gc_ssh_run_user()
{
    local USER="$1"

    highlight "Updating user [$USER]"

    # Add $USER to $RUN_GROUP
    debug "Adding [$USER] to system group [$RUN_GROUP]"
    SUSE_VERSION=""
    if [ -e '/etc/SuSE-release' ]; then
        SUSE_VERSION="$(sed -n 's/VERSION = //p' /etc/SuSE-release)"
    fi
    if [[ "$SUSE_VERSION" == '11' ]]; then
        if ! id -Gn "$USER" | grep &>/dev/null "$RUN_GROUP"; then
        ALL_GROUPS=$(groups $USER | sed "s/.*: //;s/ /,/g"),$RUN_GROUP
        # SUSE 11 requires all group names to be listed - there is no append 
        usermod -G "$ALL_GROUPS" "$USER"
    fi
    else
        usermod -a -G "$RUN_GROUP" "$USER"
    fi
    if id -Gn "$USER" | grep &>/dev/null "$RUN_GROUP"; then
        debug "User [$USER] belongs to group [$RUN_GROUP]"
    else
        die "User [$USER] does not belong to group [$RUN_GROUP]"
    fi

    local SSLRE="^ssl"
    local BITS=(${P4PORT//:/ })
    if [[ ${BITS[0]} =~ $SSLRE ]]; then
        debug "Trusting Helix server as [$USER]"
        su - "$USER" -c "p4 -p $P4PORT trust -f -y >/dev/null 2>&1"
        if [ $? -ne 0 ]; then
            error "Unable to trust the server $P4PORT"
            return 1
        fi
    fi

    # Stash the home directory
    local EXISTING_SSH_USER_HOME="$(getent passwd "$USER" | cut -d ":" -f 6 )"
    highlight "Setting file ownership and permissions for [$EXISTING_SSH_USER_HOME]"
    chown -R "$USER":"$USER_GROUP" "$EXISTING_SSH_USER_HOME"
    USER_GROUP="$(groups "$USER" | cut -d ":" -f 2 | awk '{ print $1 }')"
    chgrp -R "$USER_GROUP" "$EXISTING_SSH_USER_HOME"
    if [ -f "$EXISTING_SSH_USER_HOME/.bashrc" ]; then
        chmod ug+r "$EXISTING_SSH_USER_HOME/.bashrc"
    fi
    if [ -f "$EXISTING_SSH_USER_HOME/.bash_profile" ]; then
      chmod ug+r "$EXISTING_SSH_USER_HOME/.bash_profile"
    fi
    if [ -f "$EXISTING_SSH_USER_HOME/.bash_history" ]; then
        chmod ug+r "$EXISTING_SSH_USER_HOME/.bash_history"
    fi
    if [ -f "$EXISTING_SSH_USER_HOME/.p4enviro" ]; then
        chmod ug+r "$EXISTING_SSH_USER_HOME/.p4enviro"
    fi
    if [ -f "$EXISTING_SSH_USER_HOME/.gitconfig" ]; then
        chmod ug+r "$EXISTING_SSH_USER_HOME/.gitconfig"
    fi
    highlight "Updating GCONN_CONFIG file with path to authorized_keys"
    sed -i "s;\"authKeysFile\":[^/]*;\"authKeysFile\": \"$EXISTING_SSH_USER_HOME/.ssh/authorized_keys\",;" "$GCONNCONFIGPATH"

    highlight "User [$USER] updated!"
}

upgrade()
{
    #
    # Check if GCONN_CONFIG already exists
    #
    if [ -e "$GCONNCONFIGPATH" ]; then
        highlight "GCONN_CONFIG file already exists in [$GCONNCONFIGPATH]."
        highlight "Upgrade of GitConnector assumes that HTTPS/SSH authentication has been configured correctly."
        highlight "During the upgrade, any values passed in the command line are ignored."
        highlight "Alternatively, you can choose to overwrite existing configuration."
        if $INTERACTIVE; then
          promptfor DO_UPGRADES "Would you like to perform the upgrade of this GitConnector instance?" "no" false validate_yesno
            if [ "$DO_UPGRADES" == "no" ]; then
                promptfor REDO_CONFIG "Would you like to overwrite the existing configuration?" "no" false validate_yesno
            fi
        fi

        if [ "$DO_UPGRADES" == "no" ] && [ "$REDO_CONFIG" == "no" ]; then
            die "This GitConnector will not be upgraded and the existing configuration will not be overwritten."
        fi

        if [ "$REDO_CONFIG" == "yes" ]; then
          highlight "Moving existing configuration file to $GCONNCONFIGPATH.bak"
          mv -f "$GCONNCONFIGPATH" "$GCONNCONFIGPATH".bak
          return
        fi

        highlight "You have chosen to upgrade this instance of GitConnector."
        read_config $GCONNCONFIGPATH

        if [ -z $P4PORT ]; then
            debug "Setting P4PORT to \"$OLD_P4PORT\""
            P4PORT=$OLD_P4PORT
        fi

        if ! check_perforce_server; then
            highlight "P4PORT \"$P4PORT\" will not be used."
            die "This GitConnector will not be upgraded."
        fi

        if [ -z $GCUSER ]; then
            debug "Setting GitConnector service user to \"$OLD_GCUSER\""
            GCUSER=$OLD_GCUSER
        fi

        if [ -z $MIN_DISK_SPACE ]; then
            debug "Setting minDiskSpace to \"$OLD_MIN_DISK_SPACE\""
            MIN_DISK_SPACE=$OLD_MIN_DISK_SPACE
        fi

        debug "Logging in as \"$GCUSER\"... P4TICKECTS=$P4TICKETSPATH"
        if ! P4TRUST=$P4TRUST P4TICKETS=$P4TICKETSPATH p4 -p $P4PORT -u $GCUSER -C $P4CHARSET login -s > /dev/null 2>&1; then
            highlight "Unable to verify P4TICKETS \"$P4TICKETSPATH\"."
            highlight "Please overwrite the existing configuration."
            die "This GitConnector will not be upgraded."
        fi

        # If OLD_SERVER_ID is not null or wasn't passed in the command line,
        # set the current server id to it
        # Otherwise, use the default SERVER_ID
        if [ "$OLD_SERVER_ID" != 'null' ] && ! $SERVER_ID_FLAG; then
            SERVER_ID=$OLD_SERVER_ID
        fi

        debug "Validating server ID [$SERVER_ID]"
        if ! p4 -p "$P4PORT" -u "$GCUSER" -C "$P4CHARSET" server --exists -o $SERVER_ID > /dev/null 2>&1 ; then
          highlight "It looks like this GitConnector instance with server id [$SERVER_ID] was either"
          highlight "not registered with Helix, or registered with a non-default server id."
          highlight "If you know the server id of this GitConnector instance, you can re-run this"
          highlight "script with [--serverid <serverid_name>] argument."
          highlight "Alternatively, we recommend you choose to overwrite the existing configuration."
          die "This GitConnector will not be upgraded."
        fi

        # Check that none of the config values are null
	MISSING_CFG=""
        if [ -z "$GCLOGTIME" ] ; then MISSING_CFG+=" GCLOGTIME" ; fi
        if [ -z "$GCLOGLEVEL" ] ; then MISSING_CFG+=" GCLOGLEVEL" ; fi
        if [ -z "$REPOSDIR" ] ; then MISSING_CFG+=" REPOSDIR" ; fi
        if [ -z "$GCUSER" ] ; then MISSING_CFG+=" GCUSER" ; fi
        if [ -z "$P4PORT" ] ; then MISSING_CFG+=" P4PORT" ; fi
        if [ -z "$P4TICKETSPATH" ] ; then MISSING_CFG+=" P4TICKETSPATH" ; fi
        if [ -z "$P4TRUSTPATH" ] ; then MISSING_CFG+=" P4TRUSTPATH" ; fi
        if [ -z "$AUTHKEYSFILEPATH" ] ; then MISSING_CFG+=" AUTHKEYSFILEPATH" ; fi
        if [ -z "$ENVCONFIGPATH" ] ; then MISSING_CFG+=" ENVCONFIGPATH" ; fi
        if [ -z "$GIT_PATH" ] ; then MISSING_CFG+=" GIT_PATH" ; fi
        if [ -z "$RUN_GROUP" ] ; then MISSING_CFG+=" RUN_GROUP" ; fi
        if [ -z "$SERVER_ID" ] ; then MISSING_CFG+=" SERVER_ID" ; fi
        if [ -z "$P4GCLOGTIME" ] ; then MISSING_CFG+=" P4GCLOGTIME" ; fi
        if [ -z "$P4GCLOGLEVEL" ] ; then MISSING_CFG+=" P4GCLOGLEVEL" ; fi
        if [ ! -z "$MISSING_CFG" ] ; then
              highlight "One or more of required configuration values is missing."
              highlight "Values are Missing for:${MISSING_CFG}."
              highlight "Please overwrite the existing configuration."
              die "This GitConnector will not be upgraded."
        fi

        # Now, save a copy of the old configuration
        highlight "Moving existing configuration file to $GCONNCONFIGPATH.bak"
        mv -f "$GCONNCONFIGPATH" "$GCONNCONFIGPATH".bak

        write_config

        UPGRADE_SUCCESSFUL=true
    fi
    return
}

usage()
{
    cat <<EOS
Configuration script for Helix GitConnector.

Usage:

    $SCRIPT [-n] [-m] [--p4port <[ssl:][host:]port>]
        [--super <username>] [--superpassword <password>] [--gcuser <username>]
        [--gcuserp4password <password>] [--https] [--gconnhost <hostname>]
        [--forcehttps] [--ssh] [--sshuser <username>]
        [--homesshuser </path/to/dir>] [--croninterval <minutes>]
        [--serverid <serverid_name>] [--upgrade] [--redoconfig]
        [--mindiskspace <gigabytes>]

    $SCRIPT

Description:

    For configuring a new GitConnector instance, provide the required data via
    arguments or prompts. Configuration of either --https or --ssh is required.

    -n
            Non-interactive mode; exits immediately if prompting is required.

    -m
            Monochrome; no colored text.

    --p4port <p4port>
            The P4PORT for the Helix server.

    --super <username>
            Helix super-user's username.

    --superpassword <password>
            Helix super-user's password.

    --graphdepot <depotname>
            The name of a new Git depot to be created.

    --gcuser <username>
            GitConnector Helix's username.

    --gcuserp4password <password>
            Password set on GitConnector Helix user.

    --https
            Configure HTTPS authentication using Apache.

    --gconnhost <hostname>
            Hostname for this GitConnector. This argument is required only
            for HTTPS configuration. Suggested value is "$(hostname)".

    --forcehttps
            Force configuration of HTTPS authentication in non-interactive mode.

    --ssh
            Configure SSH authentication.

    --sshuser <username>
            The operating system user for GitConnector's SSH authentication.
            If the user does not exist, it will be created. Defaults to "git".

    --homesshuser </path/to/dir>
            Path to HOME directory of the system user used for GitConnector's
            SSH authentication. Defaults to "/home/git".

    --croninterval <minutes>
            Number between 1 and 60, an interval for how often GitConnector
            will update SSH keys if SSH authentication is chosen. Defaults to
            "10", which means GitConnector will update SSH keys every 10 minutes.
            To disable crontab, use "0".

    --serverid <serverid>
            Server ID used to identify this GitConnector instance. Defaults to
            "gconn-$(hostname)", where "$(hostname)" is the current hostname.

    --mindiskspace <gigabytes>
            Minimum disk space required for cached-repos so that remote 
            "git pull/push" operations can proceed.

    --upgrade
            Perform upgrade of this GitConnector instance.

    --redoconfig
            Reconfigure this GitConnector instance.

    --debug
            Enable debugging reporting.

    --help
            Display this help message.
EOS
}


# Validate depot name
# Depot name must be set and must start with a letter or a number
# Usage: validate_depotname <depot>
validate_depotname()
{
  local DEPOT="^[0-9a-zA-Z]+"
  if [ ! -n "$1" ] || [[ ! "$1" =~ $DEPOT ]]; then
      $hideErrors || error "Depot name must start with a letter or a number."
      return 1
  fi
  if [ -n "$GRAPH_DEPOTS" ]; then
    for depot in "${GRAPH_DEPOTS[@]}"; do
      if [ "$depot" == "$1" ]; then
        $hideErrors || error "Git depot '$1' already exists"
        return 1
      fi
    done
  fi
  return 0
}

validate_mindiskspace()
{
  local MIN_DS="^[1-9][0-9]*$"
  if [ ! -n "$1" ] || [[ ! "$1" =~ $MIN_DS ]]; then
      $hideErrors || error "MIN_DISK_SPACE must be a number greater than or equal to 1."
      return 1
  fi
  return 0
}

# Validate that response is not empty
validate_nonempty()
{
    [[ ! -n "$1" ]] && echo "-empty response given" && return 1
    return 0
}

# Validates that P4PORT looks reasonable
# If this is for binding, do some extra checks
# Usage: validate_p4port <p4port>
validate_p4port()
{
    local PORT=$1
    local BIND=false

    local PROTOS="tcp tcp4 tcp6 tcp46 tcp64 ssl ssl4 ssl6 ssl46 ssl64"
    local PROTO=""
    local HOST=""
    local PNUM=""

    local BITS=(${PORT//:/ })
    local COUNT=${#BITS[@]}
    if [ $COUNT -eq 1 ]; then
        PNUM=${BITS[0]}
    elif [ $COUNT -eq 2 ]; then
        [[ $PROTOS =~ ${BITS[0]} ]] && PROTO=${BITS[0]} || HOST=${BITS[0]}
        PNUM=${BITS[1]}
    elif [ $COUNT -eq 3 ]; then
        PROTO=${BITS[0]}
        HOST=${BITS[1]}
        PNUM=${BITS[2]}
    elif [ $COUNT -gt 3 ]; then
        $hideErrors || error "Too many parts in P4PORT: $PORT"
    fi

    #check for protocol (does it match our list of valid protocols?)
    if [ -n "$PROTO" ] && [[ ! $PROTOS =~ $PROTO ]]; then
        $hideErrors || error "Invalid Helix protocol: $PROTO"
        return 1
    fi

    # check port range (port >= 1024 && port =< 65535)
    # see http://www.iana.org/assignments/port-numbers for details
    local NUMRE="^[0-9]+$"
    if [[ ! $PNUM =~ $NUMRE ]] || [ $PNUM -lt 1024 -o $PNUM -gt 65535 ]; then
        $hideErrors || error "Port number out of range (1024-65535): $PNUM"
        return 1
    fi

    #check port not in use
    if $BIND && ! p4 -p $PNUM info 2>&1 | grep -q "Connect to server failed"; then
        $hideErrors || error "Port appears to be in use"
        return 1
    fi

    return 0
}

# Validate P4USER
# User must be set and must not start with a number
# Usage: validate_p4user <p4user>
validate_p4user()
{
    local USERRE="^[a-zA-Z]+"
    if [ ! -n "$1" ] || [[ ! "$1" =~ $USERRE ]]; then
        $hideErrors || error "Username must start with a letter."
        return 1
    fi
    return 0
}

# Checks to see if a user exists and that they have a valid home directory
validate_system_user()
{
    local USER="$1"
    if [ -z "$USER" ]; then
        error "User not specified!"
        return 1
    fi

    # Check that our user does not exist
    if ! id -nG "$USER" > /dev/null  2>&1 ; then
      debug "$USER does not exist"
      return 1
    fi

    EXISTING_SSH_USER_HOME="$(getent passwd "$USER" | cut -d ":" -f 6 )"
    debug "User [$USER] has home directory: [$EXISTING_SSH_USER_HOME]"
    if [ "$EXISTING_SSH_USER_HOME" != "$SSH_USER_HOME" ]; then
        highlight "Supplied HOME directory for the system user [$USER] is [$SSH_USER_HOME],\n\
however [$USER] exists with the HOME directory in [$EXISTING_SSH_USER_HOME].\n\
System user [$USER]'s existing HOME directory will not be changed."
    fi

    if [ -z "$EXISTING_SSH_USER_HOME" ]; then
        return 1
    fi
    if [ ! -d "$EXISTING_SSH_USER_HOME" ]; then
        return 1
    fi

    return 0
}

# Validates a yes/no prompt
# Usage: validate_yesno [yes/no>
validate_yesno()
{
    local CHOICE="$1"
    if [ -z "$CHOICE" ] || [ "$CHOICE" != "yes" -a "$CHOICE" != "no" ]; then
        $hideErrors || error "Please specify either yes or no"
        return 1
    fi
    return 0
}

write_config()
{
    highlight "Writing GitConnector configuration file"
    if [ -z "$P4PORT" -o -z "$GCUSER" -o -z "$GCONNCONFIGPATH" ]; then
        die "No P4PORT, GCUSER or GCONNCONFIGPATH specified"
    fi

    local GIT_PATH="$(dirname $(which git))"
    debug "GIT_PATH is [$GIT_PATH]"

    if [ -e  "$GCONNCONFIGPATH" ]; then
        echo "GCONN_CONFIG file already exists in [$GCONNCONFIGPATH]."
        die "Please remove this file and re-run the script."
    else
      cat >"$GCONNCONFIGPATH" <<EOT
{
    "gconn": {
        "log": {
            "levels": [
                "time=$GCLOGTIME",
                "gconn=$GCLOGLEVEL"
            ]
        },
        "reposDir": "$REPOSDIR",
        "p4User": "$GCUSER",
        "p4Port": "$P4PORT",
        "p4TicketsFile": "$P4TICKETSPATH",
        "minDiskSpace": "$MIN_DISK_SPACE",
        "p4TrustFile": "$P4TRUSTPATH",
        "authKeysFile": "$AUTHKEYSFILEPATH",
        "gitExecPath": "$GIT_PATH",
        "envPath": "$ENVCONFIGPATH",
        "authGroup": "$RUN_GROUP",
        "serverId": "$SERVER_ID"
    },
    "p4gc": {
        "log": {
            "levels": [
                "time=$P4GCLOGTIME",
                "gconn=$P4GCLOGLEVEL"
            ]
        }
    }
}
EOT
    fi
}

write_relogin_script(){
    debug "Writing re-login script to [$RELOGINSCRIPT]"
    cat >"$RELOGINSCRIPT" <<EOT
#! /bin/bash -e

if [ \$EUID != 0 ]; then
    echo "This script must run with root privileges. Attempting to sudo!"
    sudo -H "$RELOGINSCRIPT"
    exit $?
fi

echo "Performing login for [$GCUSER] on P4PORT [$P4PORT]"
P4TRUST=$P4TRUST P4TICKETS=$P4TICKETSPATH p4 -p "$P4PORT" -u "$GCUSER" -C "$P4CHARSET" login

echo "Changing permissions on [$P4TICKETSPATH]"
chmod 0440 $P4TICKETSPATH

echo "Changing group for [$P4TICKETSPATH]"
chgrp $RUN_GROUP $P4TICKETSPATH

echo "User [$GCUSER] was successfully logged in!"

EOT
    chmod +x "$RELOGINSCRIPT"
}

write_ssh_auth_script(){
    debug "Writing ssh auth script to [$GCONNSSHAUTHSCRIPT]"
    cat >"$GCONNSSHAUTHSCRIPT" <<EOT
#! /bin/bash -e
GCONN_CONFIG=/opt/perforce/git-connector/gconn.conf /opt/perforce/git-connector/bin/gconn get-ssh-key \$1 \$2
EOT
    chmod 550 "$GCONNSSHAUTHSCRIPT"
    chgrp "$RUN_GROUP" "$GCONNSSHAUTHSCRIPT"
}
#-------------------------------------------------------------------------------
# Begin functionallity
#-------------------------------------------------------------------------------

# Prevent warnings from sudo if we are in a directory the target user
# does not have permission to be in. But store the original directory first.
SCRIPT="$(basename "$0")"
SCRIPTPATH="$( cd "$(dirname "$0")" ; pwd -P )"
cd /

# Do not require sudo for --help.
if [ "$1" == "--help" ]; then
    usage
    exit 0
fi

# Compatibility checks
ensure_64bit
ensure_root "$@"

# Clean the environment
ENVREG="^P4"
for c in $(env | cut -d '=' -f 1); do
    if [[ "$c" =~ $ENVREG ]]; then
        unset "$c";
    fi;
done

# Evaluate arguments.

# P4 vars
P4USER=""
P4PASSWD=""
P4PORT=""
SERVER=""
P4CHARSET="none"
P4CASEHANDLING="unknown"
P4_SERVICETYPE=""

# GConn vars
GIT_WARN=false
HTTPS=false
FORCE_HTTPS=false
HTTPS_FAIL=false
SSH=false
SSH_FAIL=false
SSH_USER="git"
SSH_USER_HOME="/home/git"
CRON_INTERVAL=""
GRAPH_DEPOTS=()
REGISTERED_SERVICE=false
SERVER_ID=""
DEFAULT_MIN_DISK_SPACE=10
MIN_DISK_SPACE="$DEFAULT_MIN_DISK_SPACE";
SERVER_ID_FLAG=false
DO_UPGRADES="no"
REDO_CONFIG="no"
UPGRADE_SUCCESSFUL=false

# GConn vars not supplied by users
GCONNINSTALLPATH=/opt/perforce/git-connector
GCLOGTIME="1"
GCLOGLEVEL="3"
REPOSDIR="$GCONNINSTALLPATH/repos"
P4GCLOGTIME="1"
P4GCLOGLEVEL="3"
GCONNCONFIGPATH="$GCONNINSTALLPATH/gconn.conf"
P4TICKETSPATH="$GCONNINSTALLPATH/.p4tickets"
P4TRUSTPATH="$GCONNINSTALLPATH/.p4trust"
GCUSER="gconn-user"
RELOGINSCRIPT="$GCONNINSTALLPATH/bin/login-$GCUSER.sh"
GCUSER_GROUP="gconn-group"
RUN_GROUP="gconn-auth"
HTTPS_CONF_SCRIPT="$GCONNINSTALLPATH/bin/configure-https-auth.sh"
AUTHKEYSFILEPATH="none"
ENVCONFIGPATH="/usr/bin:/usr/local/bin:$GCONNINSTALLPATH/bin"
# Each component of the sshd AuthorizedKeysCommand must be owned by root.
GCONNSSHAUTHSCRIPT="/usr/bin/gconn-ssh-auth.sh"

INTERACTIVE=true

if [[ "$*" =~ '--debug' ]]; then
    echo "called with: $SCRIPT $*"
fi

TEMP=$(getopt -n "$SCRIPT" \
        -o "nm" \
        -l "p4port:,super:,superpassword:,graphdepot:,mindiskspace:,gcuser:,\
        gcuserp4password:,https,forcehttps,ssh,sshuser:,homesshuser:,\
        croninterval:,serverid:,gconnhost:,upgrade,redoconfig,debug,\
        help" -- "$@")
if [ $? -ne 0 ]; then
    usage
    exit 1
fi

# Save the arg count to determine later whether this script was called with no arguments.
NUM_ARGS=$#

# Re-inject args from getopt, so now we know they're valid and in the
# right order.
eval set -- "$TEMP"
while true; do
    case "$1" in
    -n)                 INTERACTIVE=false; shift;;
    -m)                 MONOCHROME=true;   shift;;
    --p4port)           P4PORT=$2;         shift 2;;
    --super)            P4USER=$2;         shift 2;;
    --superpassword)    P4PASSWD=$2;       shift 2;;
    --graphdepot)       GRAPHDEPOT=$2;     shift 2;;
    --gcuser)           GCUSER=$2;         shift 2;;
    --gcuserp4password) GCUSER_PASSWD=$2;  shift 2;;
    --https)            HTTPS=true;        shift ;;
    --forcehttps)       FORCE_HTTPS=true;  shift ;;
    --ssh)              SSH=true;          shift ;;
    --sshuser)          SSH_USER=$2;       shift 2;;
    --homesshuser)      SSH_USER_HOME=$2;  shift 2;;
    --croninterval)     CRON_INTERVAL=$2;  shift 2;;
    --serverid)         SERVER_ID=$2;      shift 2;;
    --gconnhost)        GCONN_HOSTNAME=$2; shift 2;;
    --mindiskspace)     MIN_DISK_SPACE=$2; shift 2;;
    --upgrade)          DO_UPGRADES="yes"; shift ;;
    --redoconfig)       REDO_CONFIG="yes"; shift ;;
    --debug)            DEBUG=true;        shift;;
    --help)             usage ;            exit 0;;
    --) shift ; break ;;
    *) die "Command-line syntax error! Unknown option: $1" ; exit 1 ;;
    esac
done

# Set default server id in case it is not specified through command line
# SERVER_ID_FLAG variable means that users passed SERVER_ID with a flag
# and if server spec already exists, it will be overwritten
if [ -n "$SERVER_ID" ]; then
    SERVER_ID_FLAG=true
else
    SERVER_ID="gconn-$(hostname)"
fi

# Set Default GCONN_HOSTNAME to suggest to users
DEFAULT_GCONN_HOSTNAME="$(hostname -f)"
if [ -z "$DEFAULT_GCONN_HOSTNAME" ]; then
    DEFAULT_GCONN_HOSTNAME="$(hostname)"
fi

# Set to default as needed.
if   [ -z "$GCONN_HOSTNAME" ]; then
    GCONN_HOSTNAME="$DEFAULT_GCONN_HOSTNAME";
fi

# Export P4TICKETS and P4TRUST env variables
export P4TRUST="$P4TRUSTPATH"
export P4TICKETS="$P4TICKETSPATH"

get_distro
if ! find_p4; then
    die "No Helix command-line client found!"
fi

hideErrors=false

ensure_dependencies

upgrade
if $UPGRADE_SUCCESSFUL ; then
    highlight "This GitConnector instance has been successfully upgraded."
    exit 0
fi

if $INTERACTIVE || $DEBUG; then
    highlightOn
    cat <<INTERACTIVE
Summary of arguments passed:

Helix server P4PORT                [${P4PORT:-(not specified)}]
Helix super-user                   [${P4USER:-(not specified)}]
Helix super-user password          [${P4PASSWD:-(not specified)}]
New Git depot name                 [${GRAPHDEPOT:-(not specified)}]
GitConnector user                  [${GCUSER:-(not specified)}]
GitConnector user password         [${GCUSER_PASSWD:-(not specified)}]
Configure HTTPS?                   [${HTTPS:-(not specified)}]
Configure SSH?                     [${SSH:-(not specified)}]
GitConnector SSH system user       [${SSH_USER:-(not specified)}]
Home directory for SSH system user [${SSH_USER_HOME:-(not specified)}]
SSH key update interval            [${CRON_INTERVAL:-10}]
Server ID                          [${SERVER_ID:-(not specified)}]
GitConnector hostname              [${GCONN_HOSTNAME:-(not specified)}]
Minimum Disk Space                 [${MIN_DISK_SPACE:-(not specified)}]

For a list of other options, type Ctrl-C to exit, and then run:
\$ sudo $SCRIPTPATH/$SCRIPT --help

You have entered interactive configuration for GitConnector. This script will
ask a series of questions, and use your answers to configure GitConnector for
first-time use. Options passed in from the command line or automatically
discovered in the environment are presented as defaults. You may press enter
to accept them, or enter an alternative.

INTERACTIVE
    highlightOff
else
    highlightOff
fi

#
# Begin data collection and validation
#

detect_local_p4d

gather_existing_p4d_configuration

get_list_of_existing_graph_depots

prompt_for_graph_depot

prompt_for_mindiskspace

# Detect if the existing authentication method for p4d is LDAP
detect_auth

# Prompt for GCUSER
promptfor GCUSER "Enter the Helix username for Git Connector" "$GCUSER" false validate_p4user

if ! p4 -p "$P4PORT" -u "$P4USER" -C "$P4CHARSET" users -a "$GCUSER"  2>&1 | grep 'no such user' > /dev/null \
    && [ -z "$AUTH_LDAP" ]; then
    if $INTERACTIVE; then
        promptfor CHANGEGCUSERPSW "Helix user [$GCUSER] already exists. Would you like to change this user's password?" "no" false validate_yesno
        if [ "$CHANGEGCUSERPSW" == "yes" ]; then
            promptfor GCUSER_PASSWD "Please provide a strong password to be set for [$GCUSER]" " " true strong_password
        fi
    fi
else
    if ! strong_password "$GCUSER_PASSWD" && [ -z "$AUTH_LDAP" ]; then
      if $INTERACTIVE; then
          hideErrors=false
          echo "This installation creates Helix user needed by this new instance of GitConnector."
          promptfor GCUSER_PASSWD "Please provide a strong password to be set for [$GCUSER]" " " true strong_password
      else
          die "Password provided for the GitConnector user is not strong enough!"
      fi
    fi
fi

prompt_for_HTTPS_config

prompt_for_SSH_config

#
# Do stuff
#
# Exit configuration if no HTTPS or SSH authentication specified
if  ! $HTTPS && ! $SSH; then
    die "GitConnector must be configured with either HTTPS or SSH authentication!"
fi

# Register this GConn as a service with p4d
register_service

# Create GCONN_CONFIG
write_config

# Exit configuration if $GRAPHDEPOT is not defined and there are no existing depots
if [ -z "$GRAPHDEPOT" ] && [ "${#GRAPH_DEPOTS[@]}" -eq 0 ]; then
    die "GitConnector cannot be configured without Git depot"
else
    # Create Git depot
    create_graph_depot
fi

# Create GCUSER and grant right permissions to GCUSER
create_gc_user

# Create system group for gconn
create_sys_run_group

# Configure syslog for log-rotation
configure_syslog

# Now configure authentication

# Perform HTTPS configuration
if $HTTPS; then
    debug "Invoking [$HTTPS_CONF_SCRIPT] script"
    export INTERACTIVE && export DEBUG && export MONOCHROME && \
    export P4PORT && export P4TICKETSPATH && export P4TRUSTPATH && \
    export GCONNCONFIGPATH && export SCRIPTPATH && export SCRIPT && \
    export FORCE_HTTPS && export RUN_GROUP && export GCONN_HOSTNAME && \
    $HTTPS_CONF_SCRIPT
    if [ $? -ne 0 ]; then
        HTTPS_FAIL=true
        error 'Failed to set up HTTPS authentication!'
    fi
fi

# If users want to configure SSH create SSH_USER
if $SSH; then
    debug "Configuring SSH authentication"
    CREATEUSER="yes"
    while [ -z "$SSH_USER" ] || ! validate_system_user "$SSH_USER"; do
        hideErrors=false
        if $INTERACTIVE; then
            # If the user doesn't exist, ask whether we should create them.
            if [ -n "$SSH_USER" ] && ! validate_system_user "$SSH_USER"; then
                promptfor CREATEUSER "Create new system user '$SSH_USER'? [yes/no]" "yes" false validate_yesno
                if [ "$CREATEUSER" == "yes" ]; then
                    create_gc_ssh_run_user "$SSH_USER"
                else
                    SSH_FAIL=true
                    highlight "System user '$SSH_USER' will not be created. Skipping SSH configuration"
                    break
                fi
            fi
        else
            if [ "$CREATEUSER" == "yes" -a -n "$SSH_USER" ]; then
                # Just in case something goes wrong: we don't want an infinite loop
                CREATEUSER="no"
                create_gc_ssh_run_user "$SSH_USER"
            else
                die "No GitConnector system user specified!"
            fi
        fi
    done
    # Update user
    if validate_system_user "$SSH_USER"; then
        update_gc_ssh_run_user "$SSH_USER"
    fi

    if $SSHD_AUTHORIZEDKEYSCOMMAND ; then
        configure_sshd_config
    else
        configure_crontab
    fi
fi

write_relogin_script

done_all_msgs
